//+------------------------------------------------------------------+
//|                                       AllAbsoluteStrength_v1.mq4 |
//|                                Copyright  2010, TrendLaboratory |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
// List of MAs:
// MA_Method= 0: SMA        - Simple Moving Average
// MA_Method= 1: EMA        - Exponential Moving Average
// MA_Method= 2: Wilder     - Wilder Exponential Moving Average
// MA_Method= 3: LWMA       - Linear Weighted Moving Average 
// MA_Method= 4: SineWMA    - Sine Weighted Moving Average
// MA_Method= 5: TriMA      - Triangular Moving Average
// MA_Method= 6: LSMA       - Least Square Moving Average (or EPMA, Linear Regression Line)
// MA_Method= 7: SMMA       - Smoothed Moving Average
// MA_Method= 8: HMA        - Hull Moving Average by Alan Hull
// MA_Method= 9: ZeroLagEMA - Zero-Lag Exponential Moving Average
// MA_Method=10: DEMA       - Double Exponential Moving Average by Patrick Mulloy
// MA_Method=11: T3         - T3 by T.Tillson
// MA_Method=12: ITrend     - Instantaneous Trendline by J.Ehlers
// MA_Method=13: Median     - Moving Median
// MA_Method=14: GeoMean    - Geometric Mean
// MA_Method=15: REMA       - Regularized EMA by Chris Satchwell
// MA_Method=16: ILRS       - Integral of Linear Regression Slope 
// MA_Method=17: IE/2       - Combination of LSMA and ILRS 
// MA_Method=18: TriMAgen   - Triangular Moving Average generalized by J.Ehlers
// MA_Method=19: VWMA       - Volume Weighted Moving Average 
// MA_Method=20: JSmooth    - Smoothing by Mark Jurik
// List of Prices:
// Price    = 0 - Close  
// Price    = 1 - Open  
// Price    = 2 - High  
// Price    = 3 - Low  
// Price    = 4 - Median Price   = (High+Low)/2  
// Price    = 5 - Typical Price  = (High+Low+Close)/3  
// Price    = 6 - Weighted Close = (High+Low+Close*2)/4
// Price    = 7 - Heiken Ashi Close  
// Price    = 8 - Heiken Ashi Open
// Price    = 9 - Heiken Ashi High
// Price    =10 - Heiken Ashi Low
 
#property copyright "Copyright  2010, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"

#property indicator_separate_window
#property indicator_buffers   6
#property indicator_color1    LightBlue
#property indicator_width1    2 
#property indicator_color2    Tomato
#property indicator_width2    2
#property indicator_color3    Aqua
#property indicator_width3    1
#property indicator_color4    Orange
#property indicator_width4    1 
#property indicator_color5    LightGray
#property indicator_width5    1
#property indicator_style5    2 
#property indicator_color6    LightGray
#property indicator_width6    1 
#property indicator_style6    2
//---- indicator parameters
extern int     TimeFrame      =  0; // Numeric value of the TimeFrame
extern int     MathMode       =  0; // 0-RSI method; 1-Stoch method; 2-DMI method
extern int     Price          =  0; // Price = 0...10 (see List of Prices)
extern int     Length         = 10; // Period of evaluation
extern int     PreSmooth      =  5; // Period of PreSmoothing
extern int     PreSmoothMode  = 11; // PreSmooth MA's Method = 0...20 (see List of MAs) 
extern int     Smooth         =  5; // Period of smoothing
extern int     SmoothMode     = 11; // Smooth MA's Method = 0...20 (see List of MAs) 
extern int     Signal         =  5; // Period of Signal Line
extern int     SignalMode     = 11; // Signal MA's Method = 0...20 (see List of MAs) 
extern double  StrengthLevel  = 70; // Strength Level (ex.70)
extern double  WeaknessLevel  = 30; // Weakness Level (ex.30)

//---- indicator buffers
double Bulls[];
double Bears[];
double SigBulls[];
double SigBears[];
double SL[];
double WL[];
//----
double tmp[][42];
double haClose[], haOpen[], haHigh[], haLow[];
int    draw_begin, mBars, pBars, mcnt_bars;
string TF, pre_name, smt_name, sig_name, math_name;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
//---- indicator buffers mapping
   SetIndexBuffer(0,Bulls);
   SetIndexBuffer(1,Bears);
   SetIndexBuffer(2,SigBulls);
   SetIndexBuffer(3,SigBears);
   SetIndexBuffer(4,SL);
   SetIndexBuffer(5,WL);   
//---- drawing settings
   SetIndexStyle(0,DRAW_LINE);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexStyle(2,DRAW_LINE);
   SetIndexStyle(3,DRAW_LINE);
   SetIndexStyle(4,DRAW_LINE);
   SetIndexStyle(5,DRAW_LINE);  
   draw_begin=Length + PreSmooth + Smooth + Signal;
//---- indicator name
   switch(MathMode)
   {
   case 0 : math_name="RSI";  break;
   case 1 : math_name="Stoch"; break;
   case 2 : math_name="DMI"; break;
   }
   
   switch(PreSmoothMode)
   {
   case 1 : pre_name="EMA(";  break;
   case 2 : pre_name="Wilder("; break;
   case 3 : pre_name="LWMA("; break;
   case 4 : pre_name="SineWMA("; break;
   case 5 : pre_name="TriMA("; break;
   case 6 : pre_name="LSMA("; break;
   case 7 : pre_name="SMMA("; break;
   case 8 : pre_name="HMA("; break;
   case 9 : pre_name="ZeroLagEMA("; break;
   case 10: pre_name="DEMA(";  break;
   case 11: pre_name="T3(";  break;
   case 12: pre_name="InstTrend(";  break;
   case 13: pre_name="Median(";  break;
   case 14: pre_name="GeoMean("; break;
   case 15: pre_name="REMA(";  break;
   case 16: pre_name="ILRS(";  break;
   case 17: pre_name="IE/2(";  break;
   case 18: pre_name="TriMA_gen("; break;
   case 19: pre_name="VWMA("; break;
   case 20: pre_name="JSmooth("; break;
   default: PreSmoothMode=0; pre_name="SMA(";break;
   }
   
   switch(SmoothMode)
   {
   case 1 : smt_name="EMA(";  break;
   case 2 : smt_name="Wilder("; break;
   case 3 : smt_name="LWMA("; break;
   case 4 : smt_name="SineWMA("; break;
   case 5 : smt_name="TriMA("; break;
   case 6 : smt_name="LSMA("; break;
   case 7 : smt_name="SMMA("; break;
   case 8 : smt_name="HMA("; break;
   case 9 : smt_name="ZeroLagEMA("; break;
   case 10: smt_name="DEMA(";  break;
   case 11: smt_name="T3(";  break;
   case 12: smt_name="InstTrend(";  break;
   case 13: smt_name="Median(";  break;
   case 14: smt_name="GeoMean("; break;
   case 15: smt_name="REMA(";  break;
   case 16: smt_name="ILRS(";  break;
   case 17: smt_name="IE/2(";  break;
   case 18: smt_name="TriMA_gen("; break;
   case 19: smt_name="VWMA("; break;
   case 20: smt_name="JSmooth("; break;
   default: SmoothMode=0; smt_name="SMA(";break;
   }
   
   switch(SignalMode)
   {
   case 1 : sig_name="EMA(";  break;
   case 2 : sig_name="Wilder("; break;
   case 3 : sig_name="LWMA("; break;
   case 4 : sig_name="SineWMA("; break;
   case 5 : sig_name="TriMA("; break;
   case 6 : sig_name="LSMA("; break;
   case 7 : sig_name="SMMA("; break;
   case 8 : sig_name="HMA("; break;
   case 9 : sig_name="ZeroLagEMA("; break;
   case 10: sig_name="DEMA(";  break;
   case 11: sig_name="T3(";  break;
   case 12: sig_name="InstTrend(";  break;
   case 13: sig_name="Median(";  break;
   case 14: sig_name="GeoMean("; break;
   case 15: sig_name="REMA(";  break;
   case 16: sig_name="ILRS(";  break;
   case 17: sig_name="IE/2(";  break;
   case 18: sig_name="TriMA_gen("; break;
   case 19: sig_name="VWMA("; break;
   case 20: sig_name="JSmooth("; break;
   default: SignalMode=0; sig_name="SMA(";break;
   }
      
   if(TimeFrame == 0) TimeFrame = Period(); 
   
   switch(TimeFrame)
   {
   case 1     : TF = "M1" ; break;
   case 5     : TF = "M5" ; break;
   case 15    : TF = "M15"; break;
   case 30    : TF = "M30"; break;
   case 60    : TF = "H1" ; break;
   case 240   : TF = "H4" ; break;
   case 1440  : TF = "D1" ; break;
   case 10080 : TF = "W1" ; break;
   case 43200 : TF = "MN1"; break;
   } 
   
   string short_name = " ("+math_name+","+Price+","+Length+","+pre_name+PreSmooth+"),"+smt_name+Smooth+"),"+sig_name+Signal+"))";
   IndicatorShortName("AllAbsoluteStrength["+TF+"]" + short_name);
   
   SetIndexDrawBegin(0,draw_begin);
   SetIndexDrawBegin(1,draw_begin);
   SetIndexDrawBegin(2,draw_begin);
   SetIndexDrawBegin(3,draw_begin);
   SetIndexDrawBegin(4,draw_begin);
   SetIndexDrawBegin(5,draw_begin);
   
   SetIndexLabel(0,"Bulls");
   SetIndexLabel(1,"Bears");
   SetIndexLabel(2,"Signal Bulls");
   SetIndexLabel(3,"Signal Bears");     
   SetIndexLabel(4,"Strength Line");
   SetIndexLabel(5,"Weakness Line");
   

//---- initialization done
   if(TimeFrame == 0 || TimeFrame < Period()) TimeFrame = Period();
   return(0);
}
//+------------------------------------------------------------------+
//| AllAbsoluteStrength_v1                                           |
//+------------------------------------------------------------------+
int start()
{
   int      limit, y, i, shift, cnt_bars=IndicatorCounted(); 
   double   aPrice[], mMA[], bulls[], bears[], lbulls[], lbears[], mbulls[], mbears[], sbulls[], sbears[];
   double   _Point = Point * MathPow(10,Digits%2);
     
   mBars = iBars(NULL,TimeFrame);
   
   if(mBars != pBars)
   {
   ArrayResize(aPrice,mBars);
   ArrayResize(mMA,mBars);
   ArrayResize(bulls,mBars);
   ArrayResize(bears,mBars);
   ArrayResize(lbulls,mBars);
   ArrayResize(lbears,mBars);
   ArrayResize(mbulls,mBars);
   ArrayResize(mbears,mBars);
   ArrayResize(sbulls,mBars);
   ArrayResize(sbears,mBars);
   ArrayResize(tmp,mBars);
   
      if(Price > 6 && Price <= 10)
      {
      ArrayResize(haClose,mBars);
      ArrayResize(haOpen,mBars);
      ArrayResize(haHigh,mBars);
      ArrayResize(haLow,mBars);
      }
   
   pBars = mBars;
   }  
   
   if(cnt_bars<1)
   {
      for(i=0;i < Bars;i++) 
      {
      Bulls[i]=0; 
      Bears[i]=0;
      SigBulls[i]=0; 
      SigBears[i]=0;
      }  
   mcnt_bars = 1;
   }

   if(mcnt_bars > 1) mcnt_bars--;
   
   for(y=mcnt_bars-1;y<mBars;y++)
   {
      if(Price <= 6) aPrice[y] = iMA(NULL,TimeFrame,1,0,0,Price,mBars-y-1);   
      else
      if(Price > 6 && Price <= 10) aPrice[y] = HeikenAshi(TimeFrame,Price-7,mBars-y-1);
      
      
      switch(PreSmoothMode)
      {
      case 1 : mMA[y] = EMA(aPrice[y],mMA,PreSmooth,y);      break;
      case 2 : mMA[y] = Wilder(aPrice,mMA,PreSmooth,y);      break;  
      case 3 : mMA[y] = LWMA(aPrice,PreSmooth,y);            break;
      case 4 : mMA[y] = SineWMA(aPrice,PreSmooth,y);         break;
      case 5 : mMA[y] = TriMA(aPrice,PreSmooth,y);           break;
      case 6 : mMA[y] = LSMA(aPrice,PreSmooth,y);            break;
      case 7 : mMA[y] = SMMA(aPrice,mMA,PreSmooth,y);        break;
      case 8 : mMA[y] = HMA(aPrice,PreSmooth,y);             break;
      case 9 : mMA[y] = ZeroLagEMA(aPrice,mMA,PreSmooth,y);  break;
      case 10: mMA[y] = DEMA(0,aPrice[y],PreSmooth,1,y);     break;
      case 11: mMA[y] = T3(0,aPrice[y],PreSmooth,0.7,y);     break;
      case 12: mMA[y] = ITrend(aPrice,mMA,PreSmooth,y);      break;
      case 13: mMA[y] = Median(aPrice,PreSmooth,y);          break;
      case 14: mMA[y] = GeoMean(aPrice,PreSmooth,y);         break;
      case 15: mMA[y] = REMA(aPrice[y],mMA,PreSmooth,0.5,y); break;
      case 16: mMA[y] = ILRS(aPrice,PreSmooth,y);            break;
      case 17: mMA[y] = IE2(aPrice,PreSmooth,y);             break;
      case 18: mMA[y] = TriMA_gen(aPrice,PreSmooth,y);       break;
      case 19: mMA[y] = VWMA(aPrice,PreSmooth,y);            break;
      case 20: mMA[y] = JSmooth(0,aPrice[y],PreSmooth,2,y);  break;
      default: mMA[y] = SMA(aPrice,PreSmooth,y);             break;
      }
      
      if(y > 2)
      {
         if (MathMode == 0)
         {
         bulls[y] = 0.5*(MathAbs(mMA[y] - mMA[y-1]) + (mMA[y] - mMA[y-1]));
         bears[y] = 0.5*(MathAbs(mMA[y] - mMA[y-1]) - (mMA[y] - mMA[y-1]));
         }
         else  
         if (MathMode == 1)
         {
         double up = 0;      
         double dn = 10000000000;
            for(i=0;i<Length;i++)
            {   
            up = MathMax(up,iHigh(NULL,TimeFrame,mBars-y-1+i));
            dn = MathMin(dn,iLow (NULL,TimeFrame,mBars-y-1+i));
            }
         bulls[y] = mMA[y] - dn;
         bears[y] = up - mMA[y];
         }
         else   
         if (MathMode == 2)
         {
         double Hi1 = iMA(NULL,0,1,0,0,2,mBars-y-1);
         double Hi2 = iMA(NULL,0,1,0,0,2,mBars-y);
         double Lo1 = iMA(NULL,0,1,0,0,3,mBars-y-1);
         double Lo2 = iMA(NULL,0,1,0,0,3,mBars-y);
      
         bulls[y] = 0.5*(MathAbs(Hi1-Hi2)+(Hi1-Hi2));
         bears[y] = 0.5*(MathAbs(Lo2-Lo1)+(Lo2-Lo1));
      
            if (bulls[y] > bears[y]) bears[y] = 0;
            else 
            if (bulls[y] < bears[y]) bulls[y] = 0;
            else
            if (bulls[y] == bears[y]) {bulls[y] = 0; bears[y] = 0;}
         }
         
         if(bulls[y] < 0) bulls[y] = 0; else bulls[y] = bulls[y] / _Point; 
         if(bears[y] < 0) bears[y] = 0; else bears[y] = bears[y] / _Point;
         
         if(MathMode == 1) int len = 1; else len = Length; 
      
         switch(SmoothMode)
         {
         case 1 : lbulls[y] = EMA(bulls[y],lbulls,len,y);      lbears[y] = EMA(bears[y],lbears,len,y);      break;
         case 2 : lbulls[y] = Wilder(bulls,lbulls,len,y);      lbears[y] = Wilder(bears,lbears,len,y);      break;  
         case 3 : lbulls[y] = LWMA(bulls,len,y);               lbears[y] = LWMA(bears,len,y);               break;
         case 4 : lbulls[y] = SineWMA(bulls,len,y);            lbears[y] = SineWMA(bears,len,y);            break;
         case 5 : lbulls[y] = TriMA(bulls,len,y);              lbears[y] = TriMA(bears,len,y);              break;
         case 6 : lbulls[y] = LSMA(bulls,len,y);               lbears[y] = LSMA(bears,len,y);               break;
         case 7 : lbulls[y] = SMMA(bulls,lbulls,len,y);        lbears[y] = SMMA(bears,lbears,len,y);        break;
         case 8 : lbulls[y] = HMA(bulls,len,y);                lbears[y] = HMA(bears,len,y);                break;
         case 9 : lbulls[y] = ZeroLagEMA(bulls,lbulls,len,y);  lbears[y] = ZeroLagEMA(bears,lbears,len,y);  break;
         case 10: lbulls[y] = DEMA(6,bulls[y],len,1,y);        lbears[y] = DEMA(12,bears[y],len,1,y);       break;
         case 11: lbulls[y] = T3(6,bulls[y],len,0.7,y);        lbears[y] = T3(12,bears[y],len,0.7,y);       break;
         case 12: lbulls[y] = ITrend(bulls,lbulls,len,y);      lbears[y] = ITrend(bears,lbears,len,y);      break;
         case 13: lbulls[y] = Median(bulls,len,y);             lbears[y] = Median(bears,len,y);             break;
         case 14: lbulls[y] = GeoMean(bulls,len,y);            lbears[y] = GeoMean(bears,len,y);            break;
         case 15: lbulls[y] = REMA(bulls[y],lbulls,len,0.5,y); lbears[y] = REMA(bears[y],lbears,len,0.5,y); break;
         case 16: lbulls[y] = ILRS(bulls,len,y);               lbears[y] = ILRS(bears,len,y);               break;
         case 17: lbulls[y] = IE2(bulls,len,y);                lbears[y] = IE2(bears,len,y);                break;
         case 18: lbulls[y] = TriMA_gen(bulls,len,y);          lbears[y] = TriMA_gen(bears,len,y);          break;
         case 19: lbulls[y] = VWMA(bulls,len,y);               lbears[y] = VWMA(bears,len,y);               break;
         case 20: lbulls[y] = JSmooth(6,bulls[y],len,2,y);     lbears[y] = JSmooth(12,bears[y],len,2,y);    break;
         default: lbulls[y] = SMA(bulls,len,y);                lbears[y] = SMA(bears,len,y);                break;
         }

         switch(SmoothMode)
         {
         case 1 : mbulls[y] = EMA(lbulls[y],mbulls,Smooth,y);     mbears[y] = EMA(lbears[y],mbears,Smooth,y);     break;
         case 2 : mbulls[y] = Wilder(lbulls,mbulls,Smooth,y);     mbears[y] = Wilder(lbears,mbears,Smooth,y);     break;  
         case 3 : mbulls[y] = LWMA(lbulls,Smooth,y);              mbears[y] = LWMA(lbears,Smooth,y);              break;
         case 4 : mbulls[y] = SineWMA(lbulls,Smooth,y);           mbears[y] = SineWMA(lbears,Smooth,y);           break;
         case 5 : mbulls[y] = TriMA(lbulls,Smooth,y);             mbears[y] = TriMA(lbears,Smooth,y);             break;
         case 6 : mbulls[y] = LSMA(lbulls,Smooth,y);              mbears[y] = LSMA(lbears,Smooth,y);              break;
         case 7 : mbulls[y] = SMMA(lbulls,mbulls,Smooth,y);       mbears[y] = SMMA(lbears,mbears,Smooth,y);       break;
         case 8 : mbulls[y] = HMA(lbulls,Smooth,y);               mbears[y] = HMA(lbears,Smooth,y);               break;
         case 9 : mbulls[y] = ZeroLagEMA(lbulls,mbulls,Smooth,y); mbears[y] = ZeroLagEMA(lbears,mbears,Smooth,y); break;
         case 10: mbulls[y] = DEMA(18,lbulls[y],Smooth,1,y);      mbears[y] = DEMA(24,lbears[y],Smooth,1,y);      break;
         case 11: mbulls[y] = T3(18,lbulls[y],Smooth,0.7,y);      mbears[y] = T3(24,lbears[y],Smooth,0.7,y);      break;
         case 12: mbulls[y] = ITrend(lbulls,mbulls,Smooth,y);     mbears[y] = ITrend(lbears,mbears,Smooth,y);     break;
         case 13: mbulls[y] = Median(lbulls,Smooth,y);            mbears[y] = Median(lbears,Smooth,y);            break;
         case 14: mbulls[y] = GeoMean(lbulls,Smooth,y);           mbears[y] = GeoMean(lbears,Smooth,y);           break;
         case 15: mbulls[y] = REMA(lbulls[y],mbulls,Smooth,0.5,y);mbears[y] = REMA(lbears[y],mbears,Smooth,0.5,y);break;
         case 16: mbulls[y] = ILRS(lbulls,Smooth,y);              mbears[y] = ILRS(lbears,Smooth,y);              break;
         case 17: mbulls[y] = IE2(lbulls,Smooth,y);               mbears[y] = IE2(lbears,Smooth,y);               break;
         case 18: mbulls[y] = TriMA_gen(lbulls,Smooth,y);         mbears[y] = TriMA_gen(lbears,Smooth,y);         break;
         case 19: mbulls[y] = VWMA(lbulls,Smooth,y);              mbears[y] = VWMA(lbears,Smooth,y);              break;
         case 20: mbulls[y] = JSmooth(18,lbulls[y],Smooth,2,y);   mbears[y] = JSmooth(24,lbears[y],Smooth,2,y);   break;
         default: mbulls[y] = SMA(lbulls,Smooth,y);               mbears[y] = SMA(lbears,Smooth,y);               break;
         }

         switch(SignalMode)
         {
         case 1 : sbulls[y] = EMA(mbulls[y],sbulls,Signal,y);     sbears[y] = EMA(mbears[y],sbears,Signal,y);     break;
         case 2 : sbulls[y] = Wilder(mbulls,sbulls,Smooth,y);     sbears[y] = Wilder(mbears,sbears,Smooth,y);     break;  
         case 3 : sbulls[y] = LWMA(mbulls,Smooth,y);              sbears[y] = LWMA(mbears,Smooth,y);              break;
         case 4 : sbulls[y] = SineWMA(mbulls,Smooth,y);           sbears[y] = SineWMA(mbears,Smooth,y);           break;
         case 5 : sbulls[y] = TriMA(mbulls,Smooth,y);             sbears[y] = TriMA(mbears,Smooth,y);             break;
         case 6 : sbulls[y] = LSMA(mbulls,Smooth,y);              sbears[y] = LSMA(mbears,Smooth,y);              break;
         case 7 : sbulls[y] = SMMA(mbulls,sbulls,Smooth,y);       sbears[y] = SMMA(mbears,sbears,Smooth,y);       break;
         case 8 : sbulls[y] = HMA(mbulls,Smooth,y);               sbears[y] = HMA(mbears,Smooth,y);               break;
         case 9 : sbulls[y] = ZeroLagEMA(mbulls,sbulls,Smooth,y); sbears[y] = ZeroLagEMA(mbears,sbears,Smooth,y); break;
         case 10: sbulls[y] = DEMA(30,mbulls[y],Smooth,1,y);      sbears[y] = DEMA(36,mbears[y],Smooth,1,y);      break;
         case 11: sbulls[y] = T3(30,mbulls[y],Smooth,0.7,y);      sbears[y] = T3(36,mbears[y],Smooth,0.7,y);      break;
         case 12: sbulls[y] = ITrend(mbulls,sbulls,Smooth,y);     sbears[y] = ITrend(mbears,sbears,Smooth,y);     break;
         case 13: sbulls[y] = Median(mbulls,Smooth,y);            sbears[y] = Median(mbears,Smooth,y);            break;
         case 14: sbulls[y] = GeoMean(mbulls,Smooth,y);           sbears[y] = GeoMean(mbears,Smooth,y);           break;
         case 15: sbulls[y] = REMA(mbulls[y],sbulls,Smooth,0.5,y);sbears[y] = REMA(mbears[y],sbears,Smooth,0.5,y);break;
         case 16: sbulls[y] = ILRS(mbulls,Smooth,y);              sbears[y] = ILRS(mbears,Smooth,y);              break;
         case 17: sbulls[y] = IE2(mbulls,Smooth,y);               sbears[y] = IE2(mbears,Smooth,y);               break;
         case 18: sbulls[y] = TriMA_gen(mbulls,Smooth,y);         sbears[y] = TriMA_gen(mbears,Smooth,y);         break;
         case 19: sbulls[y] = VWMA(mbulls,Smooth,y);              sbears[y] = VWMA(mbears,Smooth,y);              break;
         case 20: sbulls[y] = JSmooth(30,mbulls[y],Smooth,2,y);   sbears[y] = JSmooth(36,mbears[y],Smooth,2,y);   break;
         default: sbulls[y] = SMA(mbulls,Signal,y);               sbears[y] = SMA(mbears,Signal,y);               break;
         }
         
         if(TimeFrame == Period()) 
         {
         Bulls[mBars-y-1] = mbulls[y];
         Bears[mBars-y-1] = mbears[y]; 
         SigBulls[mBars-y-1] = sbulls[y];
         SigBears[mBars-y-1] = sbears[y];
            if(StrengthLevel > 0 || WeaknessLevel > 0)
            {
            SL[mBars-y-1] = StrengthLevel/100*(mbulls[y] + mbears[y]);
            WL[mBars-y-1] = WeaknessLevel/100*(mbulls[y] + mbears[y]);
            }
         }
      }
   }
   mcnt_bars = mBars-1;
   
   if(TimeFrame > Period())
   { 
      if(cnt_bars>0) cnt_bars--;
      limit = Bars-cnt_bars+TimeFrame/Period()-1;
      
      for(shift=0,y=0;shift<limit;shift++)
      {
      if (Time[shift] < iTime(NULL,TimeFrame,y)) y++; 
      
      Bulls[shift] = mbulls[mBars-y-1];
      Bears[shift] = mbears[mBars-y-1]; 
      SigBulls[shift] = sbulls[mBars-y-1];
      SigBears[shift] = sbears[mBars-y-1];
         if(StrengthLevel > 0 || WeaknessLevel > 0)
         {
         SL[shift] = StrengthLevel/100*(mbulls[mBars-y-1] + mbears[mBars-y-1]);
         WL[shift] = WeaknessLevel/100*(mbulls[mBars-y-1] + mbears[mBars-y-1]);
         }
      }
   }  
//---- done
   return(0);
}

// MA_Method=0: SMA - Simple Moving Average
double SMA(double array[],int per,int bar)
{
   double Sum = 0;
   for(int i = 0;i < per;i++) Sum += array[bar-i];
   return(Sum/per);
}                
// MA_Method=1: EMA - Exponential Moving Average
double EMA(double price,double array[],int per,int bar)
{
   if(bar == 2) double ema = price;
   else 
   if(bar > 2) ema = array[bar-1] + 2.0/(1+per)*(price - array[bar-1]); 
   if(bar == 3) Print(" ma=",ema);
   return(ema);
}
// MA_Method=2: Wilder - Wilder Exponential Moving Average
double Wilder(double array1[],double array2[],int per,int bar)
{
   if(bar == per) double wilder = SMA(array1,per,bar);
   else 
   if(bar > per) wilder = array2[bar-1] + (array1[bar] - array2[bar-1])/per; 
   
   return(wilder);
}
// MA_Method=3: LWMA - Linear Weighted Moving Average 
double LWMA(double array[],int per,int bar)
{
   double Sum = 0;
   double Weight = 0;
   
      for(int i = 0;i < per;i++)
      { 
      Weight+= (per - i);
      Sum += array[bar-i]*(per - i);
      }
   if(Weight>0) double lwma = Sum/Weight;
   else lwma = 0; 
   return(lwma);
} 
// MA_Method=4: SineWMA - Sine Weighted Moving Average
double SineWMA(double array[],int per,int bar)
{
   double pi = 3.1415926535;
   double Sum = 0;
   double Weight = 0;
   double del = 0.5*pi/per;
     
      for(int i = 0;i < per;i++)
      { 
      Weight+= MathSin(pi*(i+1)/(per+1));
      Sum += array[bar-i]*MathSin(pi*(i+1)/(per+1)); 
      }
   if(Weight>0) double swma = Sum/Weight;
   else swma = 0; 
   return(swma);
}
// MA_Method=5: TriMA - Triangular Moving Average
double TriMA(double array[],int per,int bar)
{
   double sma[];
   int len = MathCeil((per+1)*0.5);
   ArrayResize(sma,len);
   double sum=0;
   for(int i = 0;i < len;i++) 
   {
   sma[i] = SMA(array,len,bar-i);
   sum +=sma[i];
   } 
   double trima = sum/len;
   
   return(trima);
}
// MA_Method=6: LSMA - Least Square Moving Average (or EPMA, Linear Regression Line)
double LSMA(double array[],int per,int bar)
{   
   double Sum=0;
   for(int i=per; i>=1; i--) Sum += (i-(per+1)/3.0)*array[bar-per+i];
   double lsma = Sum*6/(per*(per+1));
   return(lsma);
}
// MA_Method=7: SMMA - Smoothed Moving Average
double SMMA(double array1[],double array2[],int per,int bar)
{
   if(bar == per) double smma = SMA(array1,per,bar);
   else 
   if(bar > per)
   {
   double Sum = 0;
   for(int i = 0;i < per;i++) Sum += array1[bar-i-1];
   smma = (Sum - array2[bar-1] + array1[bar])/per;
   }
   return(smma);
}                
// MA_Method=8: HMA - Hull Moving Average by Alan Hull
double HMA(double array[],int per,int bar)
{
   double tmp[];
   int len =  MathSqrt(per);
   ArrayResize(tmp,len);
   
   if(bar == per) double hma = array[bar]; 
   else
   if(bar > per)
   {
   for(int i = 0; i < len;i++) tmp[len-i-1] = 2*LWMA(array,per/2,bar-i) - LWMA(array,per,bar-i);  
   hma = LWMA(tmp,len,len-1); 
   }  

   return(hma);
}
// MA_Method=9: ZeroLagEMA - Zero-Lag Exponential Moving Average
double ZeroLagEMA(double array1[],double array2[],int per,int bar)
{
   double alfa = 2.0/(1+per); 
   int lag = 0.5*(per - 1); 
   
   if(bar == lag) double zema = array1[bar];
   else 
   if(bar > lag) zema = alfa*(2*array1[bar] - array1[bar-lag]) + (1-alfa)*array2[bar-1];
   
   return(zema);
}
// MA_Method=10: DEMA - Double Exponential Moving Average by Patrick Mulloy
double DEMA(int num,double price,double per,double v,int bar)
{
   if(bar == 2) {double dema = price; tmp[bar][num] = dema; tmp[bar][num+1] = dema;}
   else 
   if(bar > 2) 
   {
   tmp[bar][num] = tmp[bar-1][num] + 2.0/(1+per)*(price - tmp[bar-1][num]); 
   tmp[bar][num+1] = tmp[bar-1][num+1] + 2.0/(1+per)*(tmp[bar][num] - tmp[bar-1][num+1]); 
   dema = (1+v)*tmp[bar][num] - v*tmp[bar][num+1];
   }
   return(dema);
}
// MA_Method=11: T3 by T.Tillson
double T3(int num,double price,int per,double v,int bar)
{
   double len = (per/3.0)+1;
   
   if(bar == 2) 
   {
   double T3 = price; 
   for(int k=0;k<=5;k++) tmp[bar][k] = T3;
   }
   else 
   if(bar > 2) 
   {
   double dema1 = DEMA(num,price,len,v,bar); 
   double dema2 = DEMA(num+2,dema1,len,v,bar); 
   T3 = DEMA(num+4,dema2,len,v,bar);
   }
   return(T3);
}
// MA_Method=12: ITrend - Instantaneous Trendline by J.Ehlers
double ITrend(double price[],double array[],int per,int bar)
{
   double alfa = 2.0/(per+1);
   if(bar > 7)
   double it = (alfa - alfa*alfa/4)*price[bar]+ 0.5*alfa*alfa*price[bar-1]-(alfa - 0.75*alfa*alfa)*price[bar-2]+
   2*(1-alfa)*array[bar-1] - (1-alfa)*(1-alfa)*array[bar-2];
   else
   it = (price[bar] + 2*price[bar-1]+ price[bar-2])/4;
   
   return(it);
}
// MA_Method=13: Median - Moving Median
double Median(double price[],int per,int bar)
{
   double array[];
   ArrayResize(array,per);
   
   for(int i = 0; i < per;i++) array[i] = price[bar-i];
   ArraySort(array);
   
   int num = MathRound((per-1)/2); 
   if(MathMod(per,2)>0) double median = array[num]; else median = 0.5*(array[num]+array[num+1]);
   
   return(median); 
}
// MA_Method=14: GeoMean - Geometric Mean
double GeoMean(double price[],int per,int bar)
{
   double gmean = MathPow(price[bar],1.0/per); 
   for(int i = 1; i < per;i++) gmean *= MathPow(price[bar-i],1.0/per); 
   
   return(gmean);
}
// MA_Method=15: REMA - Regularized EMA by Chris Satchwell 
double REMA(double price,double array[],int per,double lambda,int bar)
{
   double alpha =  2.0/(per + 1);
   if(bar <= 3) double rema = price;
   else 
   if(bar > 3) 
   rema = (array[bar-1]*(1+2*lambda) + alpha*(price - array[bar-1]) - lambda*array[bar-2])/(1+lambda); 
   
   return(rema);
}
// MA_Method=16: ILRS - Integral of Linear Regression Slope 
double ILRS(double price[],int per,int bar)
{
   double sum = per*(per-1)*0.5;
   double sum2 = (per-1)*per*(2*per-1)/6.0;
     
   double sum1 = 0;
   double sumy = 0;
      for(int i=0;i<per;i++)
      { 
      sum1 += i*price[bar-i];
      sumy += price[bar-i];
      }
   double num1 = per*sum1 - sum*sumy;
   double num2 = sum*sum - per*sum2;
   
   if(num2 != 0) double slope = num1/num2; else slope = 0; 
   double ilrs = slope + SMA(price,per,bar);
   
   return(ilrs);
}
// MA_Method=17: IE/2 - Combination of LSMA and ILRS 
double IE2(double price[],int per,int bar)
{
   double ie = 0.5*(ILRS(price,per,bar) + LSMA(price,per,bar));
      
   return(ie); 
}

// MA_Method=18: TriMAgen - Triangular Moving Average Generalized by J.Ehlers
double TriMA_gen(double array[],int per,int bar)
{
   int len1 = MathFloor((per+1)*0.5);
   int len2 = MathCeil((per+1)*0.5);
   double sum=0;
   for(int i = 0;i < len2;i++) sum += SMA(array,len1,bar-i);
   double trimagen = sum/len2;
   
   return(trimagen);
}

// MA_Method=19: VWMA - Volume Weighted Moving Average 
double VWMA(double array[],int per,int bar)
{
   double Sum = 0;
   double Weight = 0;
   
      for(int i = 0;i < per;i++)
      { 
      Weight+= Volume[mBars-bar-1+i];
      Sum += array[bar-i]*Volume[mBars-bar-1+i];
      }
   if(Weight>0) double vwma = Sum/Weight;
   else vwma = 0; 
   return(vwma);
} 

// MA_Method=20: JSmooth - Smoothing by Mark Jurik
double JSmooth(int num,double price,int per,double pow,int bar)
{
   double beta = 0.45*(per-1)/(0.45*(per-1)+2);
	double alpha = MathPow(beta,pow);
	if(bar == 2) {tmp[bar][num+4] = price; tmp[bar][num+0] = price; tmp[bar][num+2] = price;}
	else 
   if(bar > 2) 
   {
	tmp[bar][num+0] = (1-alpha)*price + alpha*tmp[bar-1][num+0];
	tmp[bar][num+1] = (price - tmp[bar][num+0])*(1-beta) + beta*tmp[bar-1][num+1];
	tmp[bar][num+2] = tmp[bar][num+0] + tmp[bar][num+1];
	tmp[bar][num+3] = (tmp[bar][num+2] - tmp[bar-1][num+4])*MathPow((1-alpha),2) + MathPow(alpha,2)*tmp[bar-1][num+3];
	tmp[bar][num+4] = tmp[bar-1][num+4] + tmp[bar][num+3]; 
   }
   return(tmp[bar][num+4]);
}

// HeikenAshi Price:  7 - Close,8 - Open,9 - High,10 - Low 
double HeikenAshi(int tf,int price,int bar)
{ 
   if(bar == iBars(NULL,TimeFrame)- 1) 
   {
   haClose[bar] = iClose(NULL,tf,bar);
   haOpen[bar]  = iOpen(NULL,tf,bar);
   haHigh[bar]  = iHigh(NULL,tf,bar);
   haLow[bar]   = iLow(NULL,tf,bar);
   }
   else
   {
   haClose[bar] = (iOpen(NULL,tf,bar)+iHigh(NULL,tf,bar)+iLow(NULL,tf,bar)+iClose(NULL,tf,bar))/4;
   haOpen[bar]  = (haOpen[bar+1]+haClose[bar+1])/2;
   haHigh[bar]  = MathMax(iHigh(NULL,tf,bar),MathMax(haOpen[bar], haClose[bar]));
   haLow[bar]   = MathMin(iLow(NULL,tf,bar),MathMin(haOpen[bar], haClose[bar]));
   }
   
   switch(price)
   {
   case 0: return(haClose[bar]);break;
   case 1: return(haOpen[bar]);break;
   case 2: return(haHigh[bar]);break;
   case 3: return(haLow[bar]);break;
   }
}        		